<?php
/**
*
* @package phpBB3
* @version $Id: syndication.php,v 1.2.6 2015/05/04 00:00:00 TheSa|nt & wouterv $
* @copyright (c) 2007 phpBB Group
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

/**
*/
define('IN_PHPBB', true);
$phpbb_root_path = './';
$phpEx = substr(strrchr(__FILE__, '.'), 1);
include($phpbb_root_path . 'common.' . $phpEx);
include($phpbb_root_path . 'includes/functions_display.' . $phpEx);

// Start session management
$user->session_begin();
$auth->acl($user->data);
$user->setup();

// Here you can set excluded forums by numerical forum id, multiple id's separated by a comma
$cfg['exclude_forums'] = '';

// Here you can set the default number of posts in the feed
$cfg['max_topics'] = '10';

// Parameters
$chars = request_var('chars', 0);
$count = request_var('count', 0);
$fid = request_var('fid', '');
$topics = request_var('t', '');
$type = request_var('type', '');
$uid = request_var('uid', '');

// If not set, set the output count to max_topics
$count = ($count == 0) ? $cfg['max_topics'] : $count;

// If set, limit the number of characters
$chars = ($chars > 500) ? 500 : $chars;

// Generate url
$board_url = generate_board_url();
$index_url = $board_url . '/index.' . $phpEx;
$viewtopic_url = $board_url . '/viewtopic.' . $phpEx;

// ==========================================================================================================================

function codebox($preg)
{
	return $preg[2];
}

// ==========================================================================================================================

function parse_message($row, $bbcode_options, $bbcode_uid, $bbcode_bitfield)
{
	global $board_url, $type;

	$row = generate_text_for_display($row, $bbcode_uid, $bbcode_bitfield, $bbcode_options);
	$row = preg_replace_callback("/\<dt\>(.*?)\<\/dt\>\<dd\>\<code\>(.*?)\<\/code\>\<\/dd\>/si", "codebox", $row);
	$row = str_replace('<img src="./', '<img src="' . $board_url . '/', $row);

	if ($type == 'atom')
	{
		$row = str_replace("&nbsp;", " ", $row);
	}
	else
	{
		$row = str_replace("\n", "\n<br />\n", $row); // Do NOT user nl2br
		$row = htmlspecialchars($row);
	}

	return $row;
}

// ==========================================================================================================================

// Create a date according to correct RFC standard
function format_date($timestamp)
{
   global $type;

	if ($type == 'atom')
	{
		// RFC 3339 for ATOM
		return date('Y-m-d\TH:i:sP', $timestamp);
	}
	else
	{
		// RFC 822 for RSS2
		return date('D, d M Y H:i:s O', $timestamp);
	}
}

// ==========================================================================================================================

// Compile the feed link
$query_string = getenv('QUERY_STRING');
$feed_link = (generate_board_url() . "/syndication.php" . "?" . htmlspecialchars($query_string));

// Get time, use current time
$last_build_date = mktime();

// Common feed header part
$rdf = '';
$rdf .= "<?xml version=\"1.0\" encoding=\"UTF-8\"?>";

if ($type == 'atom')
{
	// ATOM feed header part
	$rdf .= "
	<feed xmlns=\"http://www.w3.org/2005/Atom\">
	<title>".strip_tags($config['sitename'])."</title>
	<subtitle>".strip_tags($config['site_desc'])."</subtitle>
	<link href=\"$board_url\" />
	<updated>".format_date($last_build_date)."</updated>
	<id>$board_url</id>
	<link rel=\"self\" href=\"$feed_link\" />";
}
else
{
	// RSS feed header part
	$rdf .= "<rss version=\"2.0\"
	xmlns:atom=\"http://www.w3.org/2005/Atom\"
	xmlns:dc=\"http://purl.org/dc/elements/1.1/\">
		<channel>
			<atom:link href=\"$feed_link\" rel=\"self\" type=\"application/rss+xml\" />
			<title>".strip_tags($config['sitename'])."</title>
			<description>".strip_tags($config['site_desc'])."</description>
			<link>$board_url</link>
			<lastBuildDate>".format_date($last_build_date)."</lastBuildDate>";
}

// Build query based on all, first or last topics only
$sql_from = 'FROM ' . POSTS_TABLE . ' as p, ' . FORUMS_TABLE . ' as f, ' . USERS_TABLE . ' as u';
if ($topics == 'first' || $topics == 'last' || $topics == 'title')
{
	$sql_from .= ', ' . TOPICS_TABLE . ' as t';
}
if ($topics == 'first' || $topics == 'title')
{
	// Only first post of topics
	$sql_where = 'AND p.post_id = t.topic_first_post_id';
}
else if ($topics == 'last')
{
	// Only last post of topics
	$sql_where = 'AND p.post_id = t.topic_last_post_id';
}
else
{
	$sql_where = '';
}

// Exclude forums
if ($cfg['exclude_forums'])
{
	$exclude_forums = explode(',', $cfg['exclude_forums']);
	$sql_where .= (' AND ' . $db->sql_in_set('p.forum_id', $exclude_forums, TRUE));
}

//Select forums
if (!empty($fid))
{
	$select_forums = explode(',', $fid);
	$sql_where .= (sizeof($select_forums)>0 ) ? ' AND  ' . $db->sql_in_set('f.forum_id', $select_forums) : '';
}

// Select users
if (!empty($uid))
{
	$select_users = explode(',', $uid);
	$sql_where .= ' AND ' . $db->sql_in_set('u.user_id', $select_users) ;
}

// SQL posts table
$sql = 'SELECT p.post_id, p.poster_id, p.post_subject, p.post_text, p.bbcode_uid, p.bbcode_bitfield, p.topic_id, p.forum_id, p.post_time, f.forum_name, f.forum_desc_options, f.forum_password, u.username
	' . $sql_from . '
	WHERE (u.user_id = p.poster_id)

	AND (f.forum_id = p.forum_id)
	' . $sql_where . '
	ORDER BY post_time DESC';
$result = $db->sql_query_limit($sql, $count, 0);

while( $row = $db->sql_fetchrow($result))
{

	// If the user has no access or the forum is passworded, skip everything until next branch
	if ((!$auth->acl_get('f_list', $row['forum_id'])) || ($row['forum_password']))
	{
		continue;
	}

	if (empty($row['post_subject']))
	{
		$subj_sql = 'SELECT p.post_subject
		FROM ' . POSTS_TABLE . ' as p, ' . TOPICS_TABLE . ' as t
		WHERE (p.topic_id = '.$row['topic_id'] .')
		AND p.post_id = t.topic_first_post_id';

		$subj_result = $db->sql_query($subj_sql);
		$subj_row = $db->sql_fetchrow($subj_result);
		$row['post_subject'] = $subj_row['post_subject'];
		$db->sql_freeresult($result);
	}

	// If chars is specified, strip HTML tags, PHP tags and BBcode, check for minimal number of chars and end with a horizontal ellipsis
	if($chars > 0)
	{
		$chars = ($chars < 3) ? 3 : $chars;
		strip_tags($row['post_text']);
		strip_bbcode($row['post_text']);
		$row['post_text'] = ( strlen($row['post_text']) > $chars ) ? substr($row['post_text'], 0, ($chars - 3)) . '&#8230;' : $row['post_text'];
	}

	// Define feed items
	$author = $row['username'];
	$time = format_date($row['post_time']);
	$link = "$board_url/viewtopic.$phpEx" ."?". 'f=' . $row['forum_id'] . '&amp;t=' . $row['topic_id']."&amp;p=".$row['post_id']."#p".$row['post_id'];
	$identifier = "$board_url/viewtopic.$phpEx" ."?". 'f=' . $row['forum_id'] . '&amp;t=' . $row['topic_id']."&amp;p=".$row['post_id']."#p".$row['post_id'];
	$title = $row['post_subject'];
	if ($topics == 'title')
	{
		$text = '';
	}
	else
	{
		$text = parse_message($row['post_text'], $row['forum_desc_options'], $row['bbcode_uid'], $row['bbcode_bitfield']);
	}

	if ($type == 'atom')
	{
		// ATOM feed items
		$rdf .= "
		<entry>
			<title>$title</title>
			<link href=\"$link\"/>
			<id>$identifier</id>
			<author>
				<name>$author</name>
			</author>
			<published>$time</published>
			<updated>$time</updated>
			<summary type=\"xhtml\">
				<div xmlns=\"http://www.w3.org/1999/xhtml\">$text</div>
			</summary>
		</entry>";
	}
	else
	{
		// RSS feed items
		$rdf .= "
			<item>
				<dc:creator>$author</dc:creator>
				<pubDate>$time</pubDate>
				<guid>$identifier</guid>
				<link>$link</link>
				<title>$title</title>
				<description>$text</description>
			</item>";
	}

}
$db->sql_freeresult($result);

// Gzip compression
if ($config['gzip_compress'])
{
	if (@extension_loaded('zlib') && !headers_sent())
	{
		ob_start('ob_gzhandler');
	}
}

if ($type == 'atom')
{
	// ATOM feed footer
	$rdf .= "
	</feed>";

	$content_type = 'application/atom+xml';
}
else
{
	// RSS feed footer
	$rdf .= "
		</channel>
	</rss>";

	$content_type = 'application/rss+xml';
}

header ('Content-Type: ' . $content_type . '; charset=UTF-8');

// Output the feed
echo($rdf);

// Kill any SQL connection
garbage_collection();

// All done
exit_handler();

?>